/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 * C Example Program 2: CDMA One Signalling Measurements
 *
 * The following example explains how to handle primary and secondary
 * addressing and how to perform simple measurement with the CMU. In the
 * example, remote control via GPIB bus and the programming language C is used.
 *
 * The following measurement example require option :
 *		CMU B-81	CDMA Link Handler
 *		CMU-K81/K82 CDMA Software package FW >= 2.50  see below
 *		CMU-K0		DEMO Software package
 *
 *		QCP820		Qualcomm CDMA phone
 *
 * Compiler name and release(s) : LabWindows/CVI 5.0
 * Original Release 1.0 : January, 2001
 *
 *
 * Version : 1.1 09/04/01
 *					
 *					CMU FW 						: V2.51.U07
 *					CVI Base driver	RSCMU200	: V1.19
 *					CVI AMPS driver	RSCMUK2A	: V1.18
 *					CVI CDMA driver	RSCMUK8C	: V1.14
 *
 * Copyright 2001 JM GAYET RSA. All Rights Reserved.
 *
 * ----------------------------------------------------------------------------
 *                         MODIFICATON HISTORY
 * ----------------------------------------------------------------------------
 *  Date      | Author      | Description
 *    Release |     Company |            
 * ----------------------------------------------------------------------------
 *  25.01.01  | JM Gayet	| Creation (accordingly to the Operating Manual
 *       V1.0 |         RSA | Edition 04).
 *            |             | 
 * ----------------------------------------------------------------------------
 *  09.04.01  | JM Gayet	| Add Handoff to AMPS test
 *       V1.1 |         RSA | Compile project with /D_HANDOFF to perform AMPS test
 *            |             | 
 * ----------------------------------------------------------------------------
 *  31.01.03  | JM Gayet	| test example with CMU FW 3.10 and 
 *       V1.2 |         RSA | CVI driver rscmuk8c 310
 *            |             | 
 *=============================================================================
 */
#include <utility.h>
#include <visa.h>
#include <ansi_c.h>
#include <userint.h>
#include "rscmu200.h"
#ifdef _HANDOFF
	#include "rscmuk2a.h"
#endif
#include "rscmuk8c.h"  


#define  SMALL_ARR_SZ 50
#define  MS_TIM 5.0

// Pointer to global structure
struct   RSCMU_Base_Globals   *p_globalvarBase;
   
// TX Min-Max power structure
typedef struct {

   ViReal64    	PowCurr;
   ViReal64    	PowAvg;
   ViReal64    	PowMin;
   ViReal64    	PowMax;
   ViReal64    	RhoCurr;
   ViReal64    	RhoAvg;
   ViReal64    	RhoMin;
   ViReal64    	RhoMax;
   ViReal64    	MOT;  
   
} tMMPOW;

// TX Modulation overview measurement structure
typedef struct  {
	ViReal64	PowExpected;
	ViReal64	PowMeasured;
	ViReal64	FreqExpected;
	ViReal64	FreqMeasured;
	ViReal64	TimErr;
	ViReal64	Rho;
	
} tOVERVIEW;

// RX Receiver measurement structure
typedef struct {
	ViReal64 				PilotActual;
	ViReal64 				PilotPower;	
	ViReal64				FER;
	ViInt32					Errors;
	ViInt32					FramesReceived;
	ViReal64				CL;
} tRX;

// Mobile info structure
typedef struct {
   ViChar     IDType   [SMALL_ARR_SZ];
   ViChar     IDNum    [SMALL_ARR_SZ];
   ViChar     SN       [SMALL_ARR_SZ];
   ViReal64   PCL;
   ViChar     dial     [SMALL_ARR_SZ];
   ViReal64   PR;
   ViChar     CallType [SMALL_ARR_SZ];
} tMS;

typedef struct {
	ViReal64		MS_InitPower;
	ViReal64		BST_InitPower;
	ViReal64		BST_CurrPower; 
	ViInt32			MATCH;
} tOLTR;

// Base station structure
typedef struct {
    ViInt32			Netw;
    ViInt32			RFChan;
    ViReal64		PowerLev;
    ViInt32			TrafficChan;
    ViReal64		TrafficLev;
    ViReal64		PagingLev;
    ViReal64		PilotLev;
    ViReal64		SyncLev;
    ViInt32			PNOffset;
    ViInt32			FrameOffset;
    ViInt32			RATE;
    ViInt32			CallMode;
    ViInt32			NETW_SID;
    ViInt32			NETW_NID;
    ViInt32			CCH;
    ViReal64		CCH_RFLev;
    ViInt32			VCH;
    ViReal64		VCH_RFLev;    
    ViInt32			CMAC;
    ViInt32			VMAC;
    ViReal64		regperiod;

} tBST;



// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif

// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
	ViChar l_vCmuErrMsg[256];
	ViChar l_vErrBuf[512];
	ViInt32 _error;
    
	if (CmuStatus != VI_SUCCESS)
	{

		//RSCMU_Base_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
		RSCMU_CDMA_errorMessage(VI_NULL, CmuStatus, l_vCmuErrMsg);

		//Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
		sprintf (l_vErrBuf, "Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);        
		MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);
	
	return -1;
	}
    return 0;
    
Error:
    return -1;
}

int	Setup_BST( ViSession	handle);
int Setup_Measurement( ViSession	handle);

  
/************************************************************************************************/
//
// main
//
/************************************************************************************************/
int main( void ) {
 
 	// Declaration
 															
	OVERVIEW             	TX_MOD_OVER;								//
	ERROR_MOD            	TX_MOD_ERROR;								//
	tMMPOW					TX_MMPOW;
	tOVERVIEW				TX_OVER;
	tOLTR					TX_OLTR;
	tRX						RX;
	tMS						MS;

	
	ViSession            	idBASE;                						// Handle with CMU base
	ViSession            	idIS95;               						// Handle with CMU CDMA
	ViSession				idAMPS;										// Handle with CMU AMPS
	ViStatus             	cmu_status;                					// Status error from driver
	ViChar               	cmu_errmsg[1024];          					// Error Message from CMU driver
	ViChar               	buffin[RD_BUFFER_SIZE];      				// Data buffer

	int						_error;
	int						TestLoop;
	int						c;
	int						flag1;
	int						flag2;

	ViInt32					over_status;
	ViInt32					fer_status;
	ViInt32					cycle;
	ViInt32					counter;
	ViInt32					state;
	ViInt32 				Gated_Burst_Ramp;
	ViReal64 				Gated_Mean_Power;


	/************************************************************************************************/
	//
	/************************************************************************************************/
	Cls();
	printf("\nR&S CMU200 - CDMA call processing example\n");
	printf("\tOpen handle with CMU base and CMU CDMA drivers\n");
	printf("\tActive trace file\n");	
	printf("\tAssign secondary adrresses\n");		
	printf("\tSet timeout\n");	
	
	// Open Handle with the BASE driver
	cmu_status = RSCMU_Init_Base (&idBASE, 								// Handle to base           				
											RSCMU_GPIB, 				// Use GPIB interface       				
											0, 							//                          				
											20,							// CMU primary addresse     				
											RSCMU_DO_QUERY, 			// Query CMU identification 				
											RSCMU_RESET_ON,				// Reset CMU                				
											cmu_errmsg);
	
	// Error
	if ( cmu_status != 0) {
		printf("\nError\t: %s\n",cmu_errmsg);
		return( cmu_status );
	}
	
	// Store CMU FW version into global structure
	CheckCmuError(RSCMU_SetCmuVersion (idBASE, 
											RSCMU_200, 					// 
											"V3.20", 					// CMU FW release
											cmu_errmsg));
	

	// Active GPIB trace 
   	CheckCmuError(RSCMU_Trace_ON (idBASE, "RsCmu.iee", cmu_errmsg));
	
	// Assign secondary addresses
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											1, 							// Addr second. 97 
											RSCMU_RF_NSIG,  			// Map RF Analyzer
											cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											2, 							// Addr second. 98
											RSCMU_CDMA800MS_NSIG, 		// Map CDMA US 800 Non signaling
							   				cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											3, 							// Addr second. 99
											RSCMU_CDMA800MS_SIG, 		// Map CDMA US 800 signaling
											cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											4, 							// Addr second. 100
											RSCMU_CDMA1900MS_NSIG,  	// Map CDMA PCS 1900 Non signaling
							   				cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											5, 							// Addr second. 101
											RSCMU_CDMA1900MS_SIG,   	// Map CDMA PCS 1900 signaling
							   				cmu_errmsg));
#ifdef _HANDOFF
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											6, 							// Addr second 102
											RSCMU_AMPSMS_SIG, 			// Map AMPS signaling group
											cmu_errmsg));
	
#endif
	
	// Open handle with the CDMA  driver for signaling mode US cellular band
	CheckCmuError(RSCMU_Init_CDMA (idBASE, 								// 
											&idIS95, 					// Handle with CDMA driver                         				
											RSCMU_CDMA800MS_SIG, 		// IS95 800 Signaling				
											cmu_errmsg));
	
	// Set Timeout base to 6 Seconds
	cmu_status = viSetAttribute (idBASE, VI_ATTR_TMO_VALUE, 6000);
	cmu_status = viSetAttribute (idIS95, VI_ATTR_TMO_VALUE, 6000);

#ifdef _HANDOFF
	CheckCmuError(RSCMU_Init_AMPS (idBASE, 
											&idAMPS, 
											RSCMU_AMPSMS_SIG, 
											cmu_errmsg));
	
	cmu_status = viSetAttribute (idAMPS, VI_ATTR_TMO_VALUE, 6000);
#endif
	
	/************************************************************************************************/
	// Select RF path Input Output
	/************************************************************************************************/
	printf("\nSelect RF path\n");
	
   	// Select RF input and output Signalling   
   	CheckCmuError(RSCMU_CDMA_InpState (idIS95, 
   											RSCMU_INPUT_RF2, 			// RF Input 2
   											cmu_errmsg));
   										
	CheckCmuError(RSCMU_CDMA_SenseCorrLossInp (idIS95, 
											RSCMU_INPUT_RF2, 
											1.0, 						// 1.0 dB compenmsation
											cmu_errmsg));
				  	
   										
   	CheckCmuError(RSCMU_CDMA_OutpState (idIS95, 
   											RSCMU_OUTPUT_RF2, 
   											cmu_errmsg));				// RF Output 2
   
	CheckCmuError(RSCMU_CDMA_SenseCorrLossOutp(idIS95, 
											RSCMU_OUTPUT_RF2, 
											1.0,						// 1.0 dB compenmsation
											cmu_errmsg));	

#ifdef _HANDOFF
   	// Select RF input and output Signalling   
   	CheckCmuError(RSCMU_CDMA_InpState (idAMPS, 
   											RSCMU_INPUT_RF2, 			// RF Input 2
   											cmu_errmsg));
   										
	CheckCmuError(RSCMU_CDMA_SenseCorrLossInp (idAMPS, 
											RSCMU_INPUT_RF2, 
											1.0, 						// 1.0 dB compenmsation
											cmu_errmsg));
				  	
   										
   	CheckCmuError(RSCMU_CDMA_OutpState (idAMPS, 
   											RSCMU_OUTPUT_RF2, 
   											cmu_errmsg));				// RF Output 2
   
	CheckCmuError(RSCMU_CDMA_SenseCorrLossOutp(idAMPS, 
											RSCMU_OUTPUT_RF2, 
											1.0,						// 1.0 dB compenmsation
											cmu_errmsg));	

#endif

	/************************************************************************************************/
	// Configure network and base station
	/************************************************************************************************/
	printf("\nConfigure CDMA Base station & Network\n");
	if ( Setup_BST(idIS95) <0)
		goto Error;

#ifdef _HANDOFF
	
	printf("\nConfigure ANALOG Base station & Network\n");
	if ( Setup_BST_ANAG(idAMPS) <0)
		goto Error;
	
#endif
   
	/************************************************************************************************/
	// Configure measurements
	/************************************************************************************************/
	printf("\nConfigure Measurements\n");
	if( Setup_Measurement(idIS95) <0 ) 
		goto Error;
	

	// Abort DSP										
	CheckCmuError(RSCMU_CDMA_AbortOverviewMod (idIS95, cmu_errmsg));
	CheckCmuError(RSCMU_CDMA_AbortOverChanQual (idIS95, cmu_errmsg));
	CheckCmuError(RSCMU_CDMA_AbortPowMaxOut (idIS95, cmu_errmsg));
	CheckCmuError(RSCMU_CDMA_AbortPowGatedOut (idIS95, cmu_errmsg));
	CheckCmuError(RSCMU_CDMA_AbortPowOLTimeResponse (idIS95, cmu_errmsg));
	CheckCmuError(RSCMU_CDMA_AbortMod (idIS95, RSCMU_OVERVIEW, cmu_errmsg));
	CheckCmuError(RSCMU_CDMA_AbortFER (idIS95, cmu_errmsg));
   	
	/************************************************************************************************/
	// Start test
	/************************************************************************************************/

	// Turn ON signaling
	CheckCmuError(RSCMU_CDMA_ProcSigAction (idIS95, 
											RSCMU_SON, 
											cmu_errmsg));

	
	TestLoop = 1;
	do
	{

		// Set type call Loopback 8 Kbit/s
   		CheckCmuError(RSCMU_CDMA_ConfBSSignalCallMode (idIS95, 
   											RSCMU_CDMA_DL8, 			// Set Voice loopback
   											cmu_errmsg));
   		
   		// Set Frame Rate to FULL
		CheckCmuError(RSCMU_CDMA_ConfBSSignalFrameRate (idIS95, 
											RSCMU_CDMA_FULL, 			// Set full RATE
											cmu_errmsg));   											
											
		// Display Message and Query item
		printf("\n\nFor a new test \tHit 1<Enter>\nFor exit\t\thit 0<Enter>\n\t\tEnter item : ");
		scanf("%d", &c);

		if (c != 1) {
		 TestLoop = 0;
		 break;
		};

		// Query user 
		while (c = getchar() != EOF) {
		   printf("\n(1)Switch ON MS\n(2)Setup a call from MS as service available\n(3)press <CTRL>Z to continue\n");

		};

		// Setup a voice call from MS
		printf("\nPlace a call from MS\n");		    
		cmu_status = RSCMU_CDMA_Origination (idIS95, 
											10, 						// Timeout 
											cmu_errmsg);
		
		// Get CMU state
		cmu_status = RSCMU_CDMA_ProcSigState (idIS95, 
											&state, 
											cmu_errmsg);
											
		if (state != RSCMU_CDMA_STATE_VCES) {
			printf("\nMS not in a call\nError : %s \n",cmu_errmsg);
			continue;
		}
		else
			printf("MS in a voice call\n");

		
		// Release call
		printf("Release call from BTS\n");
		CheckCmuError(RSCMU_CDMA_ReleaseCallFromBST (idIS95, 
											MS_TIM, 
											cmu_errmsg));
		
		
		// Set call type loopback test 8 for measurements
   		CheckCmuError(RSCMU_CDMA_ConfBSSignalCallMode (idIS95, 
   											RSCMU_CDMA_DL8, 			// Test loopback call type SO2
   											cmu_errmsg));		
		
		
		// Call MS
		printf("\nPlace a SO2 call to MS\n");
		CheckCmuError(RSCMU_CDMA_PagingCallToMS (idIS95, 10, cmu_errmsg));
		
		// Check CMU state
		CheckCmuError(RSCMU_CDMA_ProcSigState (idIS95, &state, cmu_errmsg));
		if (state != RSCMU_CDMA_STATE_CEST ){
			printf("\nMS not in a call\nError : %s \n",cmu_errmsg);
			continue;
		}
		else
			printf("MS in a SO2 call\n");		

		
		/************************************************************************************************/
		// Measurement test start here
		/************************************************************************************************/
		// Query MS info
		CheckCmuError(RSCMU_CDMA_SenseMobileStationInfo (idIS95, 
											MS.IDType, 					// IMSI
											MS.IDNum,					// 
										   	MS.SN, 						// MS serial number
										   	&MS.PCL,					// Max power in dBm
										   	MS.dial, 					// Dialed number if call from MS
										   	&MS.PR,						// Protocol Revision
										   	MS.CallType, 
										   	cmu_errmsg));
		
	
printf("\nMS info\n\
	\tID Type\t\t\t\t: %s\n\
	\tID Number\t\t\t: %s\n\
	\tSerial Num\t\t\t: %s\n\
	\tPower Class\t\t\t: %3.0lf dBm\n\
	\tProtocol Revision\t: %6.2lf\n\
	\tCall Type\t\t\t: %s\n", 
	MS.IDType,
	MS.IDNum,
	MS.SN,
	MS.PCL,
	MS.PR,
	MS.CallType);
				
		// TX TOTAL POWER  at -73 dbm (Power Expected = Base Station Level + 73)
		CheckCmuError(RSCMU_CDMA_ReadScalOverviewMod (idIS95,
											&TX_OVER.PowExpected,		// Power expected in dBm
											&TX_OVER.PowMeasured,		// Power Measured in dBm 
											&TX_OVER.FreqExpected,		// Freq expected in Hz
											&TX_OVER.FreqMeasured,		// Freq measured in Hz
											&TX_OVER.TimErr, 			// Time error in uSec
											&TX_OVER.Rho,				// Rho Waveform Quality
											cmu_errmsg));

printf("\nTX Overview modulation\n\
	\tPower Expected\t\t: %6.2lf dBm\n\
	\tPower Measured\t\t: %6.2lf dBm\n\
	\tFrequency expected\t: %6.3lf Mhz\n\
	\tFrequency Error\t\t: %6.3lf Hz\n\
	\tTime Error\t\t\t: %5.3lf uS\n\
	\tRho\t\t\t\t\t: %5.3lf\n",
	TX_OVER.PowExpected,	
	TX_OVER.PowMeasured,	
	TX_OVER.FreqExpected,	
	TX_OVER.FreqMeasured,	
	TX_OVER.TimErr, 		
	TX_OVER.Rho);			

		
		// Set Base Station Level to -100 dBm
		CheckCmuError(RSCMU_CDMA_ConfOverMCQualityLevel (idIS95,
											-100.0,						// Total RF level base station in dBm
											-14.0,						// Traffic  channel in dB
											-7.0,						// Pilot level in dB
											cmu_errmsg));
											  	
		
		// TX MAX Output measurement
		CheckCmuError(RSCMU_CDMA_ReadScalPowMaxOut (idIS95, 
											&TX_MMPOW.PowCurr,			// Max Ouput Power Current value in dBm
											&TX_MMPOW.PowAvg, 			// Max Ouput Power Average value in dBm
											&TX_MMPOW.PowMin,			// Max Ouput Power Minimum value in dBm
									  		&TX_MMPOW.PowMax,			// Max Ouput Power Maximum value in dBm
									  		&TX_MMPOW.RhoCurr,			// Waveform Quality Current value
									  		&TX_MMPOW.RhoAvg, 			// Waveform Quality Average value
									  		&TX_MMPOW.RhoMax, 			// Waveform Quality Maximum value
									  		&TX_MMPOW.RhoMin,			// Waveform Quality Minimum value
									  		&TX_MMPOW.MOT, 
									  		cmu_errmsg));
printf("\nTX MAX output power\n\
	\tAverage value\t\t: %6.2lf dBm\n\
	\tMaximum value\t\t: %6.2lf dBm\n\
	\tMinimum value\t\t: %6.2lf dBm\n\
	\nTX Waveform Quality\n\
	\tAverage Rho\t\t\t: %6.3lf\n\
	\tMaximum Rho\t\t\t: %6.3lf\n\
	\tMinimum Rho\t\t\t: %6.3lf\n",
	TX_MMPOW.PowAvg,
	TX_MMPOW.PowMin,
	TX_MMPOW.PowMax,
	TX_MMPOW.RhoAvg,
	TX_MMPOW.RhoMax,
	TX_MMPOW.RhoMin);   

		// Read Pilot
		CheckCmuError(RSCMU_CDMA_ConfOverMCQualPilotPower (idIS95,
											&RX.PilotPower,				// Pilot level
											&RX.PilotActual,			// Pilot Reported value 
											cmu_errmsg));
printf("\nRX Pilot\n\
	\tPilot reported\t\t: %6.2lf dB\n\
	\tPilot actual\t\t: %6.2lf dB\n",
	RX.PilotPower,
	RX.PilotActual);			
		
		// TX Modulation
		CheckCmuError(RSCMU_CDMA_ReadMod (idIS95, 
											RSCMU_OVERVIEW,				// Overview Modulation measurement
											&TX_MOD_ERROR,			 
											&TX_MOD_OVER,				// Measurement structure
											cmu_errmsg));
		
printf("\nTX Overview Modulation\n\
	\tEVM       PEAK Avg\t: %6.2lf %%\n\
	\tEVM       RMS  Avg\t: %6.2lf %%\n\
	\tMagn  Err PEAK Avg\t: %6.2lf \n\
	\tMagn  Err RMS  Avg\t: %6.2lf \n\
	\tPhase Err PEAK Avg\t: %6.2lf Deg\n\
	\tPhase Err RMS  Avg\t: %6.2lf Deg\n\
	\tCarrier Feedt. Avg\t: %6.2lf dB\n\
	\tI/Q Imbalance  Avg\t: %6.2lf dB\n\
	\tFreq. Err      Avg\t: %6.2lf Hz\n\
	\tTransmit Err   Avg\t: %6.2lf Us\n\
	\tRho            Avg\t: %6.2lf \n\
	\tMS Power       Avg\t: %6.2lf dBm\n",	
   TX_MOD_OVER.EVMPeak.Average,
   TX_MOD_OVER.EVMRMS.Average,
   TX_MOD_OVER.MErrPeak.Average,
   TX_MOD_OVER.MErrRMS.Average,
   TX_MOD_OVER.PErrPeak.Average,
   TX_MOD_OVER.PErrRMS.Average,
   TX_MOD_OVER.CarrierFeedthrough.Average,
   TX_MOD_OVER.IQImbalance.Average,
   TX_MOD_OVER.FreqErr.Average,
   TX_MOD_OVER.TransmitTimeError.Average,
   TX_MOD_OVER.WaveformQuality.Average,
   TX_MOD_OVER.MSPower.Average);

		// TX GATED Power
		CheckCmuError(RSCMU_CDMA_ReadScalPowGatedOut (idIS95, 
											&Gated_Mean_Power,			// Mean power in the burst
											&Gated_Burst_Ramp,			// Ramp match 
											cmu_errmsg));
		
printf("\nTX GATED Power\n\
	\tMean Power\t\t\t: %6.2lf dBm\n\
	\tBurst Matched\t\t: %2d\n",
	Gated_Mean_Power,
	Gated_Burst_Ramp);


		// RX Test
		CheckCmuError(RSCMU_CDMA_ReadFER (idIS95, 
											&RX.FER,					// FER over 100 Frames
											&RX.Errors, 				// Number of errors
											&RX.FramesReceived,			// Number of frames received
											&RX.CL, 					// Confidence level reached
											cmu_errmsg));

printf("\nRX Receiver Quality\n\
	\tFrame Error Rate\t: %6.3lf %%\n\
	\tNumber of Errors\t: %6d\n\
	\tFrames received\t\t: %6d\n\
	\tConfidence Level\t: %6.2lf %%\n",
	RX.FER,			
	RX.Errors, 		
	RX.FramesReceived,	
	RX.CL);

		// TX and RX measurement in //   
		// Set Base Station Level to -25 dBm
		CheckCmuError(RSCMU_CDMA_ConfOverMCQualityLevel (idIS95,
											-25.0,						// Total RF level base station in dBm
											-14.0,						// Traffic  channel in dB
											-7.0,						// Pilot level in dB
											cmu_errmsg));
														
		// Set control bit to UP
		CheckCmuError(RSCMU_CDMA_ConfOverMCQualityPCBits (idIS95, 
											RSCMU_BIT_RTES,
											cmu_errmsg));
										
		// Start TX Overview modulation measurement
		CheckCmuError(RSCMU_CDMA_InitOverviewMod (idIS95, cmu_errmsg));
		
		// Start RX channel quality
		CheckCmuError(RSCMU_CDMA_InitOverChanQual (idIS95, cmu_errmsg));
		
		// init variables
		flag1 = 0;
		flag2 = 0;
		over_status = 0;
		fer_status = 0;
		
		// start loop
		do {
			
			// Check CMU status
			cmu_status = RSCMU_CDMA_ProcSigState (idIS95, &state, cmu_errmsg);
			
			// if not in a call => Error 
			if ( state != RSCMU_CDMA_STATE_CEST ) {
				strcpy(cmu_errmsg,"Call drop during call\0");
				cmu_status =  RSCMU_CDMA_ERROR_CALL_DROP;
				CheckCmuError(cmu_status);
				flag1 = 1;
				flag2 = 1;
			}
		
			if (flag1 == 0) {
	
				//	Query TX overview Modulation when status ready
				if (over_status == 5) {
					CheckCmuError(RSCMU_CDMA_FetchScalOverviewMod (idIS95, 
											&TX_OVER.PowExpected,		// Power expected in dBm
											&TX_OVER.PowMeasured,		// Power Measured in dBm 
											&TX_OVER.FreqExpected,		// Freq expected in Hz
											&TX_OVER.FreqMeasured,		// Freq measured in Hz
											&TX_OVER.TimErr, 			// Time error in uSec
											&TX_OVER.Rho,
											cmu_errmsg));
														
					flag1 = 1;
					over_status = 0;
				}
				else
					// Query the overview mod status
					CheckCmuError(RSCMU_CDMA_FetchOverviewModStatus (idIS95, 
											&over_status, 
											&cycle,
											&counter, 
											cmu_errmsg));
			
				
			}
			
			if (flag2 == 0) {
				
				//	Query RX channel quality measurements if status OK
				if (fer_status == 5)  {
					CheckCmuError(RSCMU_CDMA_FetchScalOverChanQuality (idIS95, 
											&RX.FramesReceived,
											&RX.FER, 
											cmu_errmsg));
					flag2 = 1;
					fer_status = 0;
				}
				else
					// Test the channel quality status
					CheckCmuError(RSCMU_CDMA_FetchOverChanQualStatus (idIS95, 
											&fer_status,
											&cycle,
											&counter, 
											cmu_errmsg));
			
			}
			
		// end loop
		} while (!((flag1 == 1) && (flag2 == 1))); 			
		
		
		// Set control bit to normal control
		CheckCmuError(RSCMU_CDMA_ConfOverMCQualityPCBits (idIS95, 
											RSCMU_BIT_AUTO,  
											cmu_errmsg));
										
										
		
		// Abort DSP application
		CheckCmuError(RSCMU_CDMA_AbortOverviewMod (idIS95, cmu_errmsg));
		CheckCmuError(RSCMU_CDMA_AbortOverChanQual (idIS95, cmu_errmsg));
		
printf("\nTX (Min power & RX channel quality in parallel)\n\
	\tTX Power Expected\t: %6.2lf dBm\n\
	\tTX Power Measured\t: %6.2lf dBm\n\
	\tRX FER\t\t\t\t: %6.3lf %%\n\
	\tRX Frames received\t: %6.0d\n",
	TX_OVER.PowExpected,	
	TX_OVER.PowMeasured,	
	RX.FER,
	RX.FramesReceived);
	
		// Set Base Station Level to -80 dBm
		CheckCmuError(RSCMU_CDMA_ConfOverMCQualityLevel (idIS95,
											-73.0,						// Total RF level base station in dBm
											-14.0,						// Traffic  channel in dB
											-7.0,						// Pilot level in dB
											cmu_errmsg));		
   		
   		
   		// TX OLTR Power Step Direction
   		CheckCmuError(RSCMU_CDMA_ConfPowOLTRespPStepDir (idIS95, 		// Direction UP
   											RSCMU_UP, 
   											cmu_errmsg));

   		//    Initial Base Station Power
   		CheckCmuError(RSCMU_CDMA_ConfPowOLTRespIBSPow (idIS95, 				
   											-80.0,						// OLTR Initial Base station power				 
   											cmu_errmsg));


		// TX Open Loop Time Response
		CheckCmuError(RSCMU_CDMA_ReadScalPowOLResp (idIS95, 
											&TX_OLTR.MS_InitPower,		// Initial MS power in dBm
									  		&TX_OLTR.BST_InitPower,		// Initial BST power in dBm
									  		&TX_OLTR.BST_CurrPower, 	// Current BST power in dBm
									  		&TX_OLTR.MATCH,
									  		cmu_errmsg));
				
printf("\nTX OPEN LOOP TIME RESPONSE\n\
	\t\t\tStep power up 20 dB\n\
	\tMS  Initial Power\t: %6.2lf dBm\n\
	\tBST Initial Power\t: %6.2lf dBm\n\
	\tBST Current Power\t: %6.2lf dBm\n\
	\tRamp MATCHing\t\t: %2d\n",
	TX_OLTR.MS_InitPower,
	TX_OLTR.BST_InitPower,
	TX_OLTR.BST_CurrPower, 
	TX_OLTR.MATCH);
	
   		// TX OLTR Power Step Direction
   		CheckCmuError(RSCMU_CDMA_ConfPowOLTRespPStepDir (idIS95, 		// Direction DOWN
   											RSCMU_DOWN, 
   											cmu_errmsg));

   		//    Initial Base Station Power
   		CheckCmuError(RSCMU_CDMA_ConfPowOLTRespIBSPow (idIS95, 				
   											-60.0,						// OLTR Initial Base station power				 
   											cmu_errmsg));

		// TX Open Loop Time Response
		CheckCmuError(RSCMU_CDMA_ReadScalPowOLResp (idIS95, 
											&TX_OLTR.MS_InitPower,
									  		&TX_OLTR.BST_InitPower,
									  		&TX_OLTR.BST_CurrPower, 
									  		&TX_OLTR.MATCH,
									  		cmu_errmsg));
				
printf("\nTX OPEN LOOP TIME RESPONSE\n\
	\t\t\tStep power down 20 dB\n\
	\tMS  Initial Power\t: %6.2lf dBm\n\
	\tBST Initial Power\t: %6.2lf dBm\n\
	\tBST Current Power\t: %6.2lf dBm\n\
	\tRamp MATCHing\t\t: %2d\n",
	TX_OLTR.MS_InitPower,
	TX_OLTR.BST_InitPower,
	TX_OLTR.BST_CurrPower, 
	TX_OLTR.MATCH);
	

printf("\nHAND OFF TO CHANNEL 777\n");
		CheckCmuError(RSCMU_CDMA_ProcBSSignalRFChannel (idIS95, 
											777, 						// RF Channel 777
											cmu_errmsg));
		

printf("\nSET FRAME RATE Eighth\n");		
		CheckCmuError(RSCMU_CDMA_ConfBSSignalFrameRate (idIS95, 
											RSCMU_CDMA_EIGHTH, 
											cmu_errmsg));


printf("\nSET CALL MODE Data Loop Back 13K\n");		
		CheckCmuError(RSCMU_CDMA_ProcBSSignalCallMode (idIS95, 
											RSCMU_CDMA_DL13, 
											cmu_errmsg));

#ifdef _HANDOFF
		
		// Prepare handoff
		printf("\nPrepare handoff to AMPS\n");		
		CheckCmuError(RSCMU_AMPS_ConfHandoffToAMPS (idAMPS, 
											200, 
											2, 
											-85.0, 
											2000, 
											0,
							  				cmu_errmsg));
		// Handoff to AMPS					  		
		printf("\nPerform handoff to AMPS\n");				
		CheckCmuError(RSCMU_CDMA_Handoff (idIS95, 
											idAMPS, 
											RSCMU_HANDOFF_AMPSMS, 
											10,
											cmu_errmsg));
											
											
		// test if Handoff ok
		CheckCmuError(RSCMU_AMPS_ProcSigState (idAMPS, &state, cmu_errmsg));
		
		if (state !=  RSCMU_AMPS_STATE_CEST ) 
			printf("\nMS not in an AMPS call\n");
		else
			printf("\nMS in an AMPS call\n");		
		
		// release AMPS call
		printf("\nRelease AMPS call from BST\n");		
		CheckCmuError(RSCMU_AMPS_ReleaseCallFromBST (idAMPS, cmu_errmsg));
			
		// Stop signaling AMPS
		CheckCmuError(RSCMU_AMPS_ProcSigAction (idAMPS, RSCMU_SOFF, cmu_errmsg));
		
		// Start signaling CDMA
		CheckCmuError(RSCMU_CDMA_ProcSigAction (idIS95, RSCMU_SON, cmu_errmsg));
		
										  				
		
#else
		// Release call
		printf("\nRelease call from BTS\n"); 
		CheckCmuError(RSCMU_CDMA_ReleaseCallFromBST (idIS95, 
											MS_TIM, 
											cmu_errmsg));

#endif	
		
	} while (TestLoop == 1);



	// Trace OFF
   	CheckCmuError(RSCMU_Trace_OFF (idBASE, cmu_errmsg));

	// Close drivers
	RSCMU_CDMA_Close (idIS95, cmu_errmsg);
	RSCMU_Close_Base (idBASE, cmu_errmsg);
	
	return 0;	
	
Error:
	
	// Trace OFF
   	CheckCmuError(RSCMU_Trace_OFF (idBASE, cmu_errmsg));
	
	// Close drivers   	
	RSCMU_CDMA_Close (idIS95, cmu_errmsg);
	RSCMU_Close_Base (idBASE, cmu_errmsg);
	
	return -1;
} /* end main */


/************************************************************************************************/
/************************************************************************************************/
int Setup_BST( ViSession 	handle) {
	
	// Declaration
	tBST					BST;
	ViChar               	cmu_errmsg[1024];          					// Error Message from CMU driver	
	int						_error;
	
	// Start here

    BST.Netw 		= RSCMU_CDMA_U95;
    BST.RFChan  	= 283;
    BST.PowerLev	= -50.0;
    BST.TrafficChan = 8;
    BST.TrafficLev	= -14.0;
    BST.PagingLev	= -12.0;
    BST.PilotLev	= -7.0;
    BST.SyncLev		= -16.0;
    BST.PNOffset	= 0;
    BST.FrameOffset = 0;
    BST.RATE		= RSCMU_CDMA_FULL;
    BST.CallMode	= RSCMU_CDMA_VL8;
    BST.NETW_SID	= 2;
    BST.NETW_NID	= 1;
    
	
	// Turn OFF the signaling
	CheckCmuError(RSCMU_CDMA_ProcSigAction (handle, 
											RSCMU_SOFF, 				// Signaling OFF
											cmu_errmsg));
    
    
   	// Configure NETWORK & Base station
   	// Select NETWORK
   	CheckCmuError(RSCMU_CDMA_ConfNstandard (handle, 
   											BST.Netw, 					// US cellular band 
   											cmu_errmsg));

   	// Conf Base Station
   	// Set RF channel
   	CheckCmuError(RSCMU_CDMA_ConfBSSignalRFChannel (handle, 
   											BST.RFChan,					// RF Channel
   											cmu_errmsg));
   
   	// Set base station level
   	CheckCmuError(RSCMU_CDMA_ConfBSSignalLev (handle, 
			                                BST.PowerLev,				// Base station level 
			                                BST.TrafficLev,   			// Traffic Channel level
			                                BST.PagingLev,   			// Paging channel level
			                                BST.SyncLev,   				// Synchro channel level
			                                BST.PilotLev,    			// Pilot level
			                                cmu_errmsg));
   	// Set Frame Rate
   	CheckCmuError(RSCMU_CDMA_ConfBSSignalFrameRate (handle, 
   											BST.RATE, 					// Full rate 
   											cmu_errmsg));

   	// Set type call Loopback 8 Kbit/s
   	CheckCmuError(RSCMU_CDMA_ConfBSSignalCallMode (handle, 
   											BST.CallMode,	 			// Voice loopback
   											cmu_errmsg));
   												
	// Set SID
	CheckCmuError(RSCMU_CDMA_NetIdSID (handle, 
											BST.NETW_SID, 				// SID
											cmu_errmsg));
	// Set NID									
	CheckCmuError(RSCMU_CDMA_NetIdNID (handle, 
											BST.NETW_NID, 				// NID
											cmu_errmsg));

	return(0);
	
Error:   	
	return(-1);
	
}

#ifdef _HANDOFF
/************************************************************************************************/
/************************************************************************************************/
int Setup_BST_ANAG( ViSession 	handle) {
	
	// Declaration
	tBST					BST;
	ViChar               	cmu_errmsg[1024];          					// Error Message from CMU driver	
	int						_error;
	
	// Start here

    BST.NETW_SID	= 0;
	BST.CCH			= 333;
	BST.CCH_RFLev	= -50.0;
	BST.VCH			= 200;
	BST.VCH_RFLev	= -85.0;    
	BST.CMAC		= 6;
	BST.VMAC		= 2;
	BST.regperiod	= 15.0;
	
	// Turn OFF the signaling
	CheckCmuError(RSCMU_AMPS_ProcSigAction (handle, 
											RSCMU_SOFF, 				// Signaling OFF
											cmu_errmsg));

	CheckCmuError(RSCMU_AMPS_ConfBSGlobal (handle, 
											BST.CCH, 
											BST.VCH, 
											BST.CMAC, 
											BST.VMAC, 
											BST.CCH_RFLev, 
											BST.VCH_RFLev, 
											0, 
											2000,
							 				BST.NETW_SID, 
							 				cmu_errmsg));
							 				
	CheckCmuError(RSCMU_AMPS_ConfNetTimeoutRegPeriod (handle, BST.regperiod, cmu_errmsg));
	

	return(0);
	
Error:   	
	return(-1);
	
}
#endif

/************************************************************************************************/
/************************************************************************************************/
int Setup_Measurement( ViSession	handle) {

	
	ViChar               	cmu_errmsg[1024];          					// Error Message from CMU driver
	int						_error;
	
	
   	// Configure Measurement
   	// Conf overview measurement
   	CheckCmuError(RSCMU_CDMA_ConfOverMCQualityLevel (handle, 
   											-73.0, 						//	RF base station level
   											-14.0,  					//  Traffic channel
   											-7.0,   					//	Pilot level
   											cmu_errmsg));
   
   	CheckCmuError(RSCMU_CDMA_ConfOverviewModControl (handle, 
   											RSCMU_SINGLE_SHOT, 
   											RSCMU_NONE, 
   											RSCMU_NONE, 
   											cmu_errmsg));
   
   	// Conf cquality
   	CheckCmuError(RSCMU_CDMA_ConfOverChanQualControl (handle, 
   											100,						// Number of Frames 
   											RSCMU_SINGLE_SHOT,
   											RSCMU_NONE, 
   											RSCMU_NONE, 
   											cmu_errmsg));
	// 
	CheckCmuError(RSCMU_CDMA_ConfOverCQualUpperLim (handle, 
											3.0, 						// FER 3% limit
											cmu_errmsg));
   														
   
   	// Conf MAX power measurement
   	// MAX power at -103 dBm over 3 sample measurement
   	CheckCmuError(RSCMU_CDMA_ConfPowMaxOutCont (handle, 
   											3,						// Number of samples
   											RSCMU_SINGLE_SHOT,
   											RSCMU_NONE, 
   											RSCMU_NONE, 
   											cmu_errmsg));
   	CheckCmuError(RSCMU_CDMA_ConfPowMaxOutLev (handle, 
   											-103.0, 					// Base station RF level
   											-7.4,						// Traffic channel level
   											-7.0,						// Pilot level 
   											cmu_errmsg));

   // Conf MIN power measurement
   	// 3 sample measurement
   	CheckCmuError(RSCMU_CDMA_ConfPowMinOutCont (handle, 
   											3, 							// Number of samples
   											RSCMU_SINGLE_SHOT, 
   											RSCMU_NONE,
   											RSCMU_NONE, 
   											cmu_errmsg));
   
   	// FER sensitivity measurement
   	// Test set 1
   	//  Full rate -103 dBm Traffic -15.6 dB Pilot -7.0 dB Max FER 3% 95 % level confidence expected AWGN -1.0 dB
   	CheckCmuError(RSCMU_CDMA_ConfRxqFERTSetup (handle, 
   											RSCMU_CDMA_T1, 
   											cmu_errmsg));
   	CheckCmuError(RSCMU_CDMA_ConfRxqFERCont (handle, 
   											1, 
   											100,						// Number of frames send 
   											RSCMU_SINGLE_SHOT, 
   											RSCMU_CDMA_CLEX, 			// Stop on confidence level reach 
   											RSCMU_NONE, 
   											cmu_errmsg));
   	CheckCmuError(RSCMU_CDMA_ConfRxqFERTSetupFRate (handle, 
   											1, 
   											RSCMU_CDMA_FULL,    		// Full Rate
   											cmu_errmsg));
   	CheckCmuError(RSCMU_CDMA_ConfRxqFERTSetupLev (handle, 
   											1, 
   											-103.0, 					// Base station RF level dBm
   											-15.6,						// Traffic channel level dB
   											-7.0, 						// Pilot level dB
   											cmu_errmsg));
   	CheckCmuError(RSCMU_CDMA_ConfRxqFERTSetupLimits (handle, 
   											1, 	
   											3.0,						// 3 % FER limit 
   											95.0,						// 95 % level confidence 
   											cmu_errmsg));
   	CheckCmuError(RSCMU_CDMA_ConfRxqFERTSetupImpairment (handle, 
   											1, 
   											0.0, 
   											0.0, 
   											cmu_errmsg));
   
   
   	// Conf modulation measurement \
   	//    Modulation Overview
   	//    NB of samples measurement 10 
    CheckCmuError(RSCMU_CDMA_ConfModCont (handle, 
    										RSCMU_OVERVIEW, 
    										RSCMU_SCALAR, 
    										10,							// 10 Samples for measurement
    										RSCMU_SINGLE_SHOT, 
    										RSCMU_NONE, 
    										RSCMU_NONE,
    										cmu_errmsg)); 				
	
	// Set BST config for Modulation measurements
	CheckCmuError(RSCMU_CDMA_ConfModLev (handle, 
											-103.0, 					// Base station level in dBm
											-14.0,						// Traffic channel in dB 
											-7.0, 						// Pilot level in dB
											cmu_errmsg));				/* Overview Modulation */
    
   
   	// EVM modulation
   	//    NB of sample measurement 10    
   	CheckCmuError(RSCMU_CDMA_ConfModCont (handle, 
   											RSCMU_EVM, 
   											RSCMU_SCALAR, 
   											10, 
   											RSCMU_SINGLE_SHOT, 
   											RSCMU_NONE, 
   											RSCMU_NONE, 
   											cmu_errmsg)); 				/* EVM */
	

   	// Configure Open Loop Time Response
   	//    Initial Base Station Power
   	CheckCmuError(RSCMU_CDMA_ConfPowOLTRespIBSPow (handle, 				// OLTR Initial Base station power
   											-83.0,					 
   											cmu_errmsg));
   	//    Power Step Response
   	CheckCmuError(RSCMU_CDMA_ConfPowOLTRespPowStep (handle, 
   											20.0, 						// Power Step 20 dB
   											cmu_errmsg));
   	//    Power Step Direction
   	CheckCmuError(RSCMU_CDMA_ConfPowOLTRespPStepDir (handle, 			// Direction UP
   											RSCMU_UP, 
   											cmu_errmsg));
   	//    Response Level
   	CheckCmuError(RSCMU_CDMA_ConfPowOLTRespLev (handle, 
   											-7.4, 
   											-7.0, 
   											cmu_errmsg));

	RSCMU_CDMA_ConfPowGatedOutContStats (handle, 110, 120, cmu_errmsg);
	
   	// Configure Gated power
	CheckCmuError(RSCMU_CDMA_ConfPowGatedOutCont (handle, 
											RSCMU_SCALAR,
											35,
											5,
											RSCMU_SINGLE_SHOT, 
											RSCMU_NONE,
											RSCMU_NONE, 
   											cmu_errmsg));
   											
	CheckCmuError(RSCMU_CDMA_ConfPowGatedOutLev (handle, 
											-103.0, 
											-7.4, 
											-7.0, 
											cmu_errmsg));
	
	return(0);
	
Error:   	

	
	return(-1);

}
